/* 
 JSPWiki - a JSP-based WikiWiki clone.

 Copyright (C) 2001-2005 Janne Jalkanen (Janne.Jalkanen@iki.fi)

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
 the Free Software Foundation; either version 2.1 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package com.ecyrd.jspwiki.url;

import java.io.UnsupportedEncodingException;
import java.util.Properties;

import javax.servlet.http.HttpServletRequest;

import org.apache.log4j.Logger;

import com.ecyrd.jspwiki.InternalWikiException;
import com.ecyrd.jspwiki.TextUtil;
import com.ecyrd.jspwiki.WikiContext;
import com.ecyrd.jspwiki.WikiEngine;

/**
 * Provides a way to do short URLs of the form /*.html
 * 
 * @author charlse(charlse_dev@yahoo.com.cn)
 * 
 * @since 2.2
 */
public class PostfixURLConstructor extends DefaultURLConstructor {
	static Logger log = Logger.getLogger(PostfixURLConstructor.class);

	protected String m_urlPostfix = ".html";

	public static final String PROP_POSTFIX = "jspwiki.PostfixURLConstructor.postfix";

	public void initialize(WikiEngine engine, Properties properties) {
		super.initialize(engine, properties);

		m_urlPostfix = TextUtil.getStringProperty(properties, PROP_POSTFIX,
				null);

		if (m_urlPostfix == null) {
			m_urlPostfix = ".html";
		}

		log.info("Postfix URL postfix path=" + m_urlPostfix + " (You can use "
				+ PROP_POSTFIX + " to override this)");
	}

	/**
	 * Constructs the actual URL based on the context.
	 */
	private String makeURL(String context, String name, boolean absolute) {
		String viewurl = "%n" + m_urlPostfix;

		if (absolute)
			viewurl = "%u" + "%n" + m_urlPostfix;

		if (context.equals(WikiContext.VIEW)) {
			if (name == null)
				return makeURL("%u", "", absolute); // FIXME
			return doReplacement(viewurl, name, absolute);
		} else if (context.equals(WikiContext.EDIT)) {
			return doReplacement(viewurl + "?do=Edit", name, absolute);
		} else if (context.equals(WikiContext.ATTACH)) {
			return doReplacement("%Uattach/%n", name, absolute);
		} else if (context.equals(WikiContext.INFO)) {
			return doReplacement(viewurl + "?do=PageInfo", name, absolute);
		} else if (context.equals(WikiContext.DIFF)) {
			return doReplacement(viewurl + "?do=Diff", name, absolute);
		} else if (context.equals(WikiContext.NONE)) {
			return doReplacement("%U%n", name, absolute);
		} else if (context.equals(WikiContext.UPLOAD)) {
			return doReplacement(viewurl + "?do=Upload", name, absolute);
		} else if (context.equals(WikiContext.COMMENT)) {
			return doReplacement(viewurl + "?do=Comment", name, absolute);
		} else if (context.equals(WikiContext.LOGIN)) {
			return doReplacement(viewurl + "?do=Login", name, absolute);
		} else if (context.equals(WikiContext.ERROR)) {
			return doReplacement("%UError.jsp", name, absolute);
		}
		throw new InternalWikiException("Requested unsupported context "
				+ context);
	}

	/**
	 * Constructs the URL with a bunch of parameters.
	 * 
	 * @param parameters
	 *            If null or empty, no parameters are added.
	 */
	public String makeURL(String context, String name, boolean absolute,
			String parameters) {
		if (parameters != null && parameters.length() > 0) {
			if (context.equals(WikiContext.ATTACH)
					|| context.equals(WikiContext.VIEW)) {
				parameters = "?" + parameters;
			} else {
				parameters = "&amp;" + parameters;
			}
		} else {
			parameters = "";
		}
		return makeURL(context, name, absolute) + parameters;
	}

	/**
	 * Should parse the "page" parameter from the actual request.
	 */
	public String parsePage(String context, HttpServletRequest request,
			String encoding) throws UnsupportedEncodingException {
		String name = (String) request
				.getAttribute("javax.servlet.forward.servlet_path");

		if (name == null || name.length() <= 1) {
			return null;
		}

		if (name.charAt(0) == '/') {
			name = name.substring(1);
		}
		if (name.endsWith(m_urlPostfix)) {
			name = name.substring(0, name.length() - m_urlPostfix.length());
		}
		name = TextUtil.urlDecode(name, encoding);
		log.info("parsePage  name:" + name);
		return name;
	}

	public String getForwardPage(HttpServletRequest req) {
		String jspPage = req.getParameter("do");
		if (jspPage == null)
			jspPage = "Wiki";
		log.info("getForwardPage ths jspPage name" + jspPage);
		return jspPage + ".jsp";
	}
}